<?php

namespace Drupal\render_profiler;

use Drupal\Core\Cache\CacheBackendInterface;

class RunSettings {

  protected static $id = NULL;

  protected static $settings = [];

  /**
   * Determine whether we're doing a profiling run, and initialize settings for
   * the run if so.
   *
   * @return string|false
   *   The ID string for this run, or FALSE if we're not profiling this run.
   */
  public static function initialize() {
    if (!is_null(static::$id)) {
      return static::$id;
    }
    // Check the request query to see if a profiling run has been requested.
    $params = \Drupal::request()->query;
    static::$id = $params->get('RENDER_PROFILER_START') ?? $params->get('RENDER_PROFILER_START_SILENT');
    if (empty(static::$id)) {
      static::$id = FALSE;
      return FALSE;
    }
    // Check user permissions.
    if (!\Drupal::currentUser()->hasPermission('use render_profiler')) {
      static::$id = FALSE;
      return FALSE;
    }
    // Configure settings for the run, paying attention to overriding query
    // parameters.
    $config = \Drupal::config('render_profiler.settings');
    static::set('display_table', 1);
    static::set('display_annotations', $config->get('display_annotations'));
    static::set('threshold', $config->get('threshold'));
    if ($params->has('RP_ANNOTATIONS')) {
      static::set('display_annotations', $params->get('RP_ANNOTATIONS'));
    }
    if ($params->has('RP_THRESHOLD')) {
      static::set('threshold', $params->get('RP_THRESHOLD'));
    }
    if ($params->has('RENDER_PROFILER_START_SILENT')) {
      static::set('display_table', 0);
      static::set('display_annotations', 0);
    }
  }

  /**
   * @param string $name
   *   The name of the setting.
   * @param $value
   *   The value of the setting.
   *
   * @return void
   */
  public static function set(string $name, $value) {
    static::$settings[$name] = $value;
  }

  /**
   * @param string $name
   *   The name of the setting.
   *
   * @return mixed|null
   *   The value of the setting, or NULL if not set.
   */
  public static function get(string $name) {
    return static::$settings[$name] ?? NULL;
  }

  /**
   * @return null
   */
  public static function id() {
    return static::$id;
  }

  /**
   * Stop logging and save the log results to cache.
   *
   * @return void
   */
  public static function stop() {
    // Cache the result and stop the run.
    if (!empty(static::$id)) {
      \Drupal::cache()
        ->set('render_profiler:' . static::id(), RenderProfilerLogger::getEntries(), CacheBackendInterface::CACHE_PERMANENT, ['rendered']);
      static::$id = FALSE;
    }
  }

}
